<?php
/**
 * Postmen API Orders Class
 *
 * Handles requests to the /orders endpoint
 *
 * @author      Postmen
 * @category    API
 * @package     Postmen/API
 * @since       1.0
 */

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

class Postmen_API_Orders extends Postmen_API_Resource
{

    /** @var string $base the route base */
    protected $base = '/orders';

    /**
     * Register the routes for this class
     *
     * GET /orders
     * GET /orders/count
     *
     * @param array $routes
     *
     * @return array
     * @since 2.1
     *
     */
    public function register_routes($routes)
    {
        # GET /orders/ping
        $routes[$this->base . '/ping'] = [
            [[$this, 'ping'], Postmen_API_Server::READABLE],
        ];

        # GET /orders/get_orders
        $routes[$this->base . '/get_orders'] = [
            [[$this, 'get_orders'], Postmen_API_Server::READABLE],
        ];

        # PUT /orders/update_order
        $routes[$this->base . '/update_order'] = [
            [[$this, 'update_order'], Postmen_API_Server::METHOD_PUT | Postmen_API_Server::ACCEPT_DATA],
        ];

        return $routes;
    }

    /**
     * heath checkendpoint for wordpress url validation
     *
     * @return string
     * @since 2.1
     */
    public function ping()
    {
        return 'pong';
    }

    /**
     * Get orders
     *
     * @param string $updated_at_min
     * @param string $updated_at_max
     * @param string $max_results_number
     *
     * @return array
     * @since 2.1
     *
     */
    public function get_orders($updated_at_min = null, $updated_at_max = null, $max_results_number = null)
    {
        $args = [
            'updated_at_min' => $updated_at_min,
            'updated_at_max' => $updated_at_max,
            'orderby' => 'modified',
            'order' => 'ASC',
            'limit' => $max_results_number,
            'page' => !empty($_GET['page']) && intval($_GET['page']) > 1 ? absint($_GET['page']) : 1
        ];
        $weight_unit = get_option('woocommerce_weight_unit');
        $query = $this->query_orders($args);
        $orders = [];
        foreach ($query->posts as $order_id) {
            if (!$this->is_readable($order_id)) {
                continue;
            }
            $orders[] = current($this->get_order($order_id, $weight_unit));
        }

        return $orders;
    }

    /**
     * Update a particular order
     *
     * @param $data
     *
     * @return array
     * @since 2.1
     *
     */
    public function update_order($data)
    {
        $id = $data['id'];
        $status = $data['status'];
        $note = $data['note'];
        $customer_note = $data['customer_note'];
        $tracking_slug = $data['tracking_slug'];
        $tracking_number = $data['tracking_number'];
        $actions_sync_aftership = isset($data['actions_sync_aftership']) ? $data['actions_sync_aftership'] : false;

        $id = $this->validate_request($id, 'shop_order', 'read');

        if (is_wp_error($id)) {
            return $id;
        }

        $order = new WC_Order($id);
        $post_id = $order->post->ID;

        // update order status
        if ($status) {
            // Woocommerce is quite good at guessing which status do we mean
            // eventually we can do it manually using following function
            // wc_get_order_statuses();
            $order->update_status($status);
        }
        // create order note if order note string is not empty
        if ($note) {
            $order->add_order_note($note, $customer_note);
        }

        // create aftership tracking if aftership plugin enabled
        if (is_plugin_active('aftership-woocommerce-tracking/aftership.php')) {
            if (isset($tracking_slug)) {
                update_post_meta($post_id, '_aftership_tracking_provider', wc_clean($tracking_slug));
            }

            if (isset($tracking_number) && $actions_sync_aftership == true) {
                update_post_meta($post_id, '_aftership_tracking_number',
                    wc_clean(is_array($tracking_number) ? array_shift($tracking_number) : $tracking_number));
            }

            if (function_exists('getAfterShipInstance')) {
                // surpress the output from aftership plugin to not cause exception on
                // worker side
                ob_start();
                getAfterShipInstance()->display_tracking_info($id);
                ob_end_clean();
            }
        }

        return $data;
    }

    /**
     * @param $id
     * @param $weight_unit
     * @return array|int|WP_Error
     * @throws Exception
     */
    private function get_order($id, $weight_unit)
    {
        // ensure order ID is valid & user has permission to read
        $id = $this->validate_request($id, 'shop_order', 'read');

        if (is_wp_error($id)) {
            return $id;
        }

        $order = new WC_Order($id);
        $order_post = get_post($id);
        $customer = new WC_Customer($order->get_customer_id());
        $shipping_method = current($order->get_shipping_methods());
        $order_data = [
            'id' => $order->get_id(),
            'order_number' => $order->get_order_number(),
            'order_name' => $order->get_order_number(),
            'taxes_included' => ($order->get_total_tax() > 0),
            'shipping_method' => [
                'code' => $shipping_method['method_id'],
                'name' => $shipping_method['name'],
            ],
            'created_at' => $order_post->post_date_gmt,
            'updated_at' => $order_post->post_modified_gmt,
            'status' => $order->get_status(),
            'currency' => $order->get_currency(),
            'total' => wc_format_decimal($order->get_total(), 2),
            'total_line_items_quantity' => $order->get_item_count(),
            'total_shipping' => wc_format_decimal($order->get_total_shipping(), 2),
            'shipping_methods' => $order->get_shipping_method(),
            'weight_unit' => $weight_unit,
            'total_tax' => $order->get_total_tax(),
            'locale' => get_locale(),
            'metrics' => [
                'placed_at' => $this->server->format_datetime($order->get_date_created()),
                'updated_at' => $this->server->format_datetime($order->get_date_modified()),
            ],
            'payment_details' => [
                'method_id' => $order->get_payment_method(),
                'method_title' => $order->get_payment_method_title(),
                'paid' => isset($order->paid_date),
            ],
            'customer' => [
                'id' => $order->get_customer_id(),
                'user_email' => $customer->get_email(),
                'first_name' => $customer->get_first_name(),
                'last_name' => $customer->get_last_name(),
                'emails' => ($customer->get_email()) ? [$customer->get_email()] : [],
                'phones' => ($customer->get_billing_phone()) ? [[
                    'country_code' => null,
                    'number' => $customer->get_billing_phone()
                ]] : [],
            ],

            'shipping_address' => [
                'first_name' => $order->get_shipping_first_name(),
                'last_name' => $order->get_shipping_last_name(),
                'company' => $order->get_shipping_company(),
                'address_1' => $order->get_shipping_address_1(),
                'address_2' => $order->get_shipping_address_2(),
                'city' => $order->get_shipping_city(),
                'state' => $order->get_shipping_state(),
                'postcode' => $order->get_shipping_postcode(),
                'country' => $order->get_shipping_country(),
                'email' => $order->get_billing_email(),
                'phone' => $order->get_billing_phone()
            ],
            'billing_address' => array(
                'first_name' => $order->get_billing_first_name(),
                'last_name' => $order->get_billing_last_name(),
                'company' => $order->get_billing_company(),
                'address_1' => $order->get_billing_address_1(),
                'address_2' => $order->get_billing_address_2(),
                'city' => $order->get_billing_city(),
                'state' => $order->get_billing_state(),
                'postcode' => $order->get_billing_postcode(),
                'country' => $order->get_billing_country(),
                'email' => $order->get_billing_email(),
                'phone' => $order->get_billing_phone(),
            ),
            'note' => $order->customer_note,
            'line_items' => [],
            'shipping_lines' => []
        ];


        // add line items
        foreach ($order->get_items() as $item_id => $item) {
            if (is_callable($item, 'get_product')) {
                $product = $item->get_product();
            } else {
                $product = $order->get_product_from_item($item);
            }


            if (empty($product)) {
                // if the $product is empty then the following lines will crash
                continue;
            } else {
                $weight = $product->get_weight();
            }
            $product_id = (isset($product->variation_id)) ? $product->variation_id : $product->get_id();
            // set the response object
            $terms_tags = get_the_terms($product_id, 'product_tag');
            $prudct_tags = [];
            foreach ($terms_tags as $termsKey => $termsVal) {
                $prudct_tags[] = $termsVal->name;
            }
            $prudct_categories = [];

            $categories = get_the_terms($product_id, 'product_cat');
            foreach ($categories as $categoriesKey => $categoriesVal) {
                $prudct_categories[] = $categoriesVal->name;
            }

            $order_data['line_items'][] = [
                'id' => $item_id,
                'subtotal' => wc_format_decimal($order->get_line_subtotal($item), 2),
                'total' => wc_format_decimal($order->get_line_total($item), 2),
                'price' => wc_format_decimal($order->get_item_total($item), 2),
                'quantity' => (int)$item['qty'],
                'name' => $item['name'],
                'title' => $item['name'],
                'product_id' => $product_id,
                'sku' => is_object($product) ? $product->get_sku() : null,
                'image_url' => wp_get_attachment_url($product->image_id),
                'image_urls' => wp_get_attachment_url($product->image_id) ? [wp_get_attachment_url($product->image_id)] : [],
                'weight' => $weight,
                'unit_weight' => [
                    'unit' => $weight_unit,
                    'value' => (float)$weight,
                ],
                'unit_price' => [
                    'currency' => $order->get_currency(),
                    'amount' => (float)wc_format_decimal($order->get_item_total($item), 2),
                ],
                'tags' => $prudct_tags,
                'categories' => $prudct_categories,
                'description' => get_post($product_id)->post_content ? get_post($product_id)->post_content : '',
                'refunded_quantity' => $order->get_qty_refunded_for_item( $item_id ),
            ];


        }

        // add shipping
        foreach ($order->get_shipping_methods() as $shipping_item_id => $shipping_item) {

            $order_data['shipping_lines'][] = [
                'id' => $shipping_item['method_id'],
                'method_title' => $shipping_item['name'],
                'total' => wc_format_decimal($shipping_item['cost'], 2),
            ];
        }
        // The function definition will be available after installing the aftership plugin.
        if(function_exists('order_post_meta_getter')) {
            $order_data['aftership']['woocommerce']['trackings'][0] = array(
                'tracking_provider' => "",
                'tracking_number' => "",
                'tracking_ship_date' => "",
                'tracking_postal_code' => "",
                'tracking_account_number' => "",
                'tracking_key' => "",
                'tracking_destination_country' => "",
            );

            // tracking number
            $aftership_tracking_number = order_post_meta_getter($order, 'aftership_tracking_number');
            if (!empty($aftership_tracking_number)) {
                $order_data['aftership']['woocommerce']['trackings'][0] = array(
                    'tracking_provider' => order_post_meta_getter($order, 'aftership_tracking_provider'),
                    'tracking_number' => order_post_meta_getter($order, 'aftership_tracking_number'),
                    'tracking_ship_date' => order_post_meta_getter($order, 'aftership_tracking_shipdate'),
                    'tracking_postal_code' => order_post_meta_getter($order, 'aftership_tracking_postal'),
                    'tracking_account_number' => order_post_meta_getter($order, 'aftership_tracking_account'),
                    'tracking_key' => order_post_meta_getter($order, 'aftership_tracking_key'),
                    'tracking_destination_country' => order_post_meta_getter($order, 'aftership_tracking_destination_country'),
                );
            }

            // 兼容 woocommerce 官方的 tracking 插件
            $woocommerce_tracking_arr = order_post_meta_getter($order, 'wc_shipment_tracking_items');
            if (empty($aftership_tracking_number) && !empty($woocommerce_tracking_arr)) {
                foreach ($woocommerce_tracking_arr as $trackingKey => $trackingVal) {
                    $order_data['aftership']['woocommerce']['trackings'][$trackingKey] = array(
                        'tracking_provider' => $trackingVal["tracking_provider"],
                        'tracking_number' => $trackingVal["tracking_number"],
                        'tracking_ship_date' => $trackingVal["date_shipped"],
                        'tracking_postal_code' => "",
                        'tracking_account_number' => "",
                        'tracking_key' => "",
                        'tracking_destination_country' => "",
                    );
                    $trackingArr = $this->getTrackingInfoByShipmentTracking($trackingVal);
                    if (!empty($trackingArr)) {
                        $order_data['aftership']['woocommerce']['trackings'][$trackingKey]['tracking_postal_code'] = $trackingArr['tracking_postal_code'];
                        $order_data['aftership']['woocommerce']['trackings'][$trackingKey]['tracking_provider'] = $trackingArr['tracking_provider'];
                    }
                }

            }
        }
        return ['order' => $order_data];
    }

    // 从wc ShipmentTracking 插件获取 Postalcode  - postnl
    private function getTrackingInfoByShipmentTracking($tracking_items)
    {
        if (!isset($tracking_items['custom_tracking_link'])) {
            return array();
        }

        // 获取 postnl  Postalcode
        $urlArr = parse_url(stripslashes($tracking_items['custom_tracking_link']));

        if ($urlArr === false) {
            return array();
        }

        if (!isset($urlArr['host'])) {
            return array();
        }

        $hostArr = explode(".", $urlArr['host']);
        $hostArrIndex = count($hostArr) - 2;
        if (empty($hostArr) || !isset($hostArr[$hostArrIndex])) {
            return array();
        }

        if ($hostArr[$hostArrIndex] == 'postnl') {
            parse_str($urlArr['query'], $queryArr);
            if (!isset($queryArr['Postalcode'])) {
                return array();
            }

            return array(
                'tracking_provider' => 'postnl',
                'tracking_postal_code' => str_replace(" ", "", $queryArr['Postalcode']),
            );
        }
        return array();
    }


    /**
     * Helper method to get order post objects
     *
     * @param array $args request arguments for filtering query
     *
     * @return WP_Query
     * @since 2.1
     *
     */
    private function query_orders($args)
    {
        $woo_version = postmen_wpbo_get_woo_version_number();

        if ($woo_version >= 2.2) {
            // set base query arguments
            $query_args = [
                'fields' => 'ids',
                'post_type' => 'shop_order',
                'post_status' => array_keys(wc_get_order_statuses())
            ];
        } else {
            // set base query arguments
            $query_args = [
                'fields' => 'ids',
                'post_type' => 'shop_order',
                'post_status' => 'publish',
            ];
        }

        // add status argument
        if (!empty($args['status'])) {

            $statuses = explode(',', $args['status']);

            $query_args['tax_query'] = [
                [
                    'taxonomy' => 'shop_order_status',
                    'field' => 'slug',
                    'terms' => $statuses,
                ],
            ];

            unset($args['status']);
        }

        $query_args = $this->merge_query_args($query_args, $args);

        return new WP_Query($query_args);
    }

}
